/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task;

import org.ow2.petals.ant.AbstractJBIAntTask;

/**
 * Ant task : jbi-unload-all-inst. 
 * <p>
 * This task unload all components. If the boolean "force" is set to true, components can be uninstalled in all states.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 */
public class UnloadAllInstallersTask extends AbstractJBIAntTask {

    /**
     * Boolean specifying if installer unloading must be executed in any state
     */
    private boolean force = false;

    @Override
    public void doTask() throws Exception {
        String[] componentNames = this.getJMXClient().getInstallationServiceClient()
                .unloadAllInstallers(force);

        for (String componentName : componentNames) {
            log("Installer of the component '" + componentName + "' unloaded");
        }

    }

    /**
     * set the force parameter
     * 
     * @param force
     */
    public void setForce(boolean force) {
        this.force = force;
    }

}
