/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task.monit.assertion.flowtreenode;

import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.ow2.petals.ant.task.monit.FlowTreeNode;
import org.ow2.petals.ant.task.monit.assertion.exception.BuildAssertParamEmptyException;
import org.ow2.petals.ant.task.monit.assertion.exception.BuildAssertParamMissingException;
import org.ow2.petals.ant.task.monit.assertion.flowtreenode.exception.BuildAssertFlowInstanceIdNotEqualsException;
import org.ow2.petals.ant.task.monit.assertion.flowtreenode.exception.BuildFlowTreeNodeAssertionException;

/**
 * An assertion about the flow instance identifier of a flow tree node
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class FlowInstanceIdEquals implements FlowTreeNodeAssertion {

    /**
     * The expected flow instance identifier
     */
    private String expected;

    /**
     * Set the expected flow instance identifier
     * 
     * @param expected
     *            The expected flow instance identifier. Should be not {@code null} and not empty.
     */
    public void setExpected(final String expected) {
        assert expected != null;

        this.expected = expected;
    }

    @Override
    public void doAssert(final FlowTreeNode flowTreeNode, final Project project, final Location location)
            throws BuildFlowTreeNodeAssertionException {

        if (this.expected == null) {
            throw new BuildAssertParamMissingException("expected", location);
        } else if (this.expected.isEmpty()) {
            throw new BuildAssertParamEmptyException("expected", location);
        } else if (!this.expected.equals(flowTreeNode.getFlow().getFlowId())) {
            throw new BuildAssertFlowInstanceIdNotEqualsException(flowTreeNode.getFlow().getFlowId(), this.expected,
                    location);
        }
    }

}
