/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.apache.tools.ant.BuildException;
import org.junit.Test;

/**
 * @author Adrien Ruffie - EBM WebSourcing
 */
public class AbstractConfigureArchiveAntTaskTest {

    private class TestConfigureTask extends AbstractConfigureArchiveAntTask {

        public final void validateParam(final String fileParameter, final String paramaterName)
                throws BuildException {
            AbstractConfigureArchiveAntTask.validateFileParameter(fileParameter, paramaterName);
        }

    }

    /**
     * Check
     * {@link AbstractConfigureArchiveAntTask#validateFileParameter(String, String)}
     * against a <code>null</code> parameter to validate.
     */
    @Test
    public void testValidateFileParameter_Null() {
        final TestConfigureTask tct = new TestConfigureTask();

        try {
            tct.validateParam(null, "file");
            fail("The exception '" + BuildException.class.getName() + "' is not thrown.");
        } catch (final BuildException e) {
            assertNotNull(e.getMessage());
            assertTrue(e.getMessage().contains("Missing attribute 'file'"));
        }

    }

    /**
     * Check
     * {@link AbstractConfigureArchiveAntTask#validateFileParameter(String, String)}
     * against a empty parameter to validate.
     */
    @Test
    public void testValidateFileParameter_Empty() {
        final TestConfigureTask tct = new TestConfigureTask();

        try {
            tct.validateParam("", "file");
            fail("The exception '" + BuildException.class.getName() + "' is not thrown.");
        } catch (final BuildException e) {
            assertNotNull(e.getMessage());
            assertTrue(e.getMessage().contains("Empty attribute 'file'"));
        }

    }

    /**
     * Check
     * {@link AbstractConfigureArchiveAntTask#validateFileParameter(String, String)}
     * against an invalid file.
     */
    @Test
    public void testValidateFileParameter_Invalid() {
        final String invalidFile = "/tmp/foo";
        final TestConfigureTask tct = new TestConfigureTask();

        try {
            tct.validateParam(invalidFile, "file");
            fail("The exception '" + BuildException.class.getName() + "' is not thrown.");
        } catch (final BuildException e) {
            assertNotNull(e.getMessage());
            assertTrue(e.getMessage().contains(invalidFile));
            assertTrue(e.getMessage().contains("is not a valid archive (zip or jar required)"));
        }

    }

    /**
     * Check
     * {@link AbstractConfigureArchiveAntTask#validateFileParameter(String, String)}
     * against an JAR file.
     */
    @Test
    public void testValidateFileParameter_JAR() throws BuildException {
        final TestConfigureTask tct = new TestConfigureTask();
        tct.validateParam("/tmp/foo.jar", "file");
    }

    /**
     * Check
     * {@link AbstractConfigureArchiveAntTask#validateFileParameter(String, String)}
     * against an ZIP file.
     */
    @Test
    public void testValidateFileParameter_ZIP() throws BuildException {
        final TestConfigureTask tct = new TestConfigureTask();
        tct.validateParam("/tmp/foo.zip", "file");
    }
}
