/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant;

import junit.framework.TestCase;

import org.apache.tools.ant.BuildException;
import org.ow2.petals.ant.task.ShutdownComponentTask;

/**
 * @author Mathieu Carrolle - EBM WebSourcing
 */
public class ShutdownComponentTaskTest extends TestCase {

    /**
     * Check {@link ShutdownComponentTask} against a component identifier set to
     * <code>null</code>.
     */
    public void testComponentIdentifier_Null() {
        final ShutdownComponentTask task = new ShutdownComponentTask();
        task.setName(null);

        try {
            task.execute();
            TestCase.fail("The exception '" + BuildException.class.getName() + "' is not thrown.");
        } catch (final BuildException e) {
            TestCase.assertNotNull(e.getMessage());
            TestCase.assertTrue(e.getMessage().contains("Missing attribute 'name'"));
        }
    }

    /**
     * Check {@link ShutdownComponentTask} against an empty component
     * identifier.
     */
    public void testComponentIdentifier_Empty() {
        final ShutdownComponentTask task = new ShutdownComponentTask();
        task.setName("");

        try {
            task.execute();
            TestCase.fail("The exception '" + BuildException.class.getName() + "' is not thrown.");
        } catch (final BuildException e) {
            TestCase.assertNotNull(e.getMessage());
            TestCase.assertTrue(e.getMessage().contains("Empty attribute 'name'"));
        }
    }
}
