/**
 * Copyright (c) 2015-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task.monit.assertion.flowinstance;

import java.net.URISyntaxException;

import org.apache.tools.ant.Project;
import org.junit.Test;
import org.ow2.petals.ant.task.monit.FlowStepInfo;
import org.ow2.petals.ant.task.monit.assertion.flowstep.AbstractAssertAttributeTest;
import org.ow2.petals.ant.task.monit.assertion.flowstep.AssertFlowStep;
import org.ow2.petals.ant.task.monit.assertion.flowstep.AttributeEquals;
import org.ow2.petals.ant.task.monit.assertion.flowstep.AttributeSet;
import org.ow2.petals.ant.task.monit.assertion.flowstep.exception.BuildAssertFlowStepEndException;
import org.ow2.petals.ant.task.monit.assertion.flowstep.exception.BuildFlowInstanceIdInvalidException;
import org.ow2.petals.ant.task.monit.exception.BuildFlowStepIdxInvalidException;
import org.ow2.petals.ant.task.monit.exception.BuildPropertyMissingException;
import org.ow2.petals.ant.task.monit.exception.BuildRefIdInvalidException;
import org.ow2.petals.commons.log.FlowLogData;
import org.ow2.petals.commons.log.TraceCode;

/**
 * Unit tests of {@link AssertFlowInstanceTask}. Unit tests of nested assertions are in package
 * <code>org.ow2.petals.ant.task.monit.assertion.flowstep</code>.
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class AssertFlowInstanceTaskTest extends AbstractAssertAttributeTest {

    /**
     * Check assertions on flow steps of a given flow instance read from MONIT traces under nominal conditions
     */
    @Test
    public void nominal() throws URISyntaxException {

        final Project project = this.loadMonitTraces();

        final AssertFlowStep assertFlowStep_0 = new AssertFlowStep();
        assertFlowStep_0.setFlowStepIdx(0);
        final AttributeEquals attributeEquals_0_traceCode = new AttributeEquals();
        attributeEquals_0_traceCode.setName(FlowLogData.FLOW_STEP_TRACE_CODE);
        attributeEquals_0_traceCode.setValue(TraceCode.CONSUME_EXT_FLOW_STEP_BEGIN.toString());
        assertFlowStep_0.addAttributeEquals(attributeEquals_0_traceCode);
        final AttributeEquals attributeEquals_0_flowInstanceId = new AttributeEquals();
        attributeEquals_0_flowInstanceId.setName(FlowLogData.FLOW_INSTANCE_ID_PROPERTY_NAME);
        attributeEquals_0_flowInstanceId.setValue(FLOW_INSTANCE_ID);
        assertFlowStep_0.addAttributeEquals(attributeEquals_0_flowInstanceId);
        final AttributeSet attributeSet_0_flowStepId = new AttributeSet();
        attributeSet_0_flowStepId.setName(FlowLogData.FLOW_STEP_ID_PROPERTY_NAME);
        assertFlowStep_0.addAttributeSet(attributeSet_0_flowStepId);
        final AttributeSet attributeSet_0_correlatedFlowInstanceId = new AttributeSet();
        attributeSet_0_correlatedFlowInstanceId.setName("correlatedFlowInstanceId");
        assertFlowStep_0.addAttributeSet(attributeSet_0_correlatedFlowInstanceId);
        final AttributeSet attributeSet_0_correlatedFlowStepId = new AttributeSet();
        attributeSet_0_correlatedFlowStepId.setName("correlatedFlowStepId");
        assertFlowStep_0.addAttributeSet(attributeSet_0_correlatedFlowStepId);
        final FlowStepInfo flowStepInfo_0_flowStepId = new FlowStepInfo();
        flowStepInfo_0_flowStepId.setProperty("1st-flowStepId");
        flowStepInfo_0_flowStepId.setAttribute(FlowLogData.FLOW_STEP_ID_PROPERTY_NAME);
        assertFlowStep_0.addFlowStepInfo(flowStepInfo_0_flowStepId);

        final AssertFlowInstanceTask afitTask_0 = new AssertFlowInstanceTask();
        afitTask_0.setRefid(REF_ID);
        afitTask_0.setFlowInstanceId(FLOW_INSTANCE_ID);
        afitTask_0.addAssertFlowStep(assertFlowStep_0);
        afitTask_0.setProject(project);

        afitTask_0.execute();

        // To check flow step info usage, it is needed to run the task twice in this unit test
        final AssertFlowStep assertFlowStep_1 = new AssertFlowStep();
        assertFlowStep_1.setFlowStepIdx(1);
        final AttributeEquals attributeEquals_1_traceCode = new AttributeEquals();
        attributeEquals_1_traceCode.setName(FlowLogData.FLOW_STEP_TRACE_CODE);
        attributeEquals_1_traceCode.setValue(TraceCode.PROVIDE_FLOW_STEP_BEGIN.toString());
        assertFlowStep_1.addAttributeEquals(attributeEquals_1_traceCode);
        final AttributeEquals attributeEquals_1_flowInstanceId = new AttributeEquals();
        attributeEquals_1_flowInstanceId.setName(FlowLogData.FLOW_INSTANCE_ID_PROPERTY_NAME);
        attributeEquals_1_flowInstanceId.setValue(FLOW_INSTANCE_ID);
        assertFlowStep_1.addAttributeEquals(attributeEquals_1_flowInstanceId);
        final AttributeEquals attributeEquals_1_flowPreviousStepId = new AttributeEquals();
        attributeEquals_1_flowPreviousStepId.setName(FlowLogData.FLOW_PREVIOUS_STEP_ID_PROPERTY_NAME);
        attributeEquals_1_flowPreviousStepId.setValue(project.getProperty("1st-flowStepId"));
        assertFlowStep_1.addAttributeEquals(attributeEquals_1_flowPreviousStepId);

        final AssertFlowInstanceTask afitTask_1 = new AssertFlowInstanceTask();
        afitTask_1.setRefid(REF_ID);
        afitTask_1.setFlowInstanceId(FLOW_INSTANCE_ID);
        afitTask_1.addAssertFlowStep(assertFlowStep_1);
        afitTask_1.setProject(project);

        afitTask_1.execute();
    }

    /**
     * Check assertions on flow steps of a given flow instance ending unexpectedly with error
     */
    @Test(expected = BuildAssertFlowStepEndException.class)
    public void unexpectedErrorOnFlowStep() throws URISyntaxException {

        final Project project = this.loadMonitTraces();

        final AssertFlowStep assertFlowStep_0 = new AssertFlowStep();
        assertFlowStep_0.setFlowStepIdx(1);

        final AssertFlowInstanceTask afitTask_0 = new AssertFlowInstanceTask();
        afitTask_0.setRefid(REF_ID);
        afitTask_0.setFlowInstanceId("ee30a530-378d-11e5-9ac3-0090f5fbc4a1");
        afitTask_0.addAssertFlowStep(assertFlowStep_0);
        afitTask_0.setProject(project);

        afitTask_0.execute();
    }

    /**
     * Check assertions on flow steps of a given flow instance ending unexpectedly with success
     */
    @Test(expected = BuildAssertFlowStepEndException.class)
    public void unexpectedSuccessOnFlowStep() throws URISyntaxException {

        final Project project = this.loadMonitTraces();

        final AssertFlowStep assertFlowStep_0 = new AssertFlowStep();
        assertFlowStep_0.setFlowStepIdx(1);
        assertFlowStep_0.setEndsWithError(true);

        final AssertFlowInstanceTask afitTask_0 = new AssertFlowInstanceTask();
        afitTask_0.setRefid(REF_ID);
        afitTask_0.setFlowInstanceId(FLOW_INSTANCE_ID);
        afitTask_0.addAssertFlowStep(assertFlowStep_0);
        afitTask_0.setProject(project);

        afitTask_0.execute();
    }

    /**
     * Check assertions on flow steps of a given flow instance ending with failure
     */
    @Test
    public void nominalWithFlowStepFailure() throws URISyntaxException {

        final Project project = this.loadMonitTraces();

        final AssertFlowStep assertFlowStep_0 = new AssertFlowStep();
        assertFlowStep_0.setFlowStepIdx(1);
        assertFlowStep_0.setEndsWithError(true);

        final AssertFlowInstanceTask afitTask_0 = new AssertFlowInstanceTask();
        afitTask_0.setRefid(REF_ID);
        afitTask_0.setFlowInstanceId("ee30a530-378d-11e5-9ac3-0090f5fbc4a1");
        afitTask_0.addAssertFlowStep(assertFlowStep_0);
        afitTask_0.setProject(project);

        afitTask_0.execute();
    }

    /**
     * Check assertions on flow steps of a given flow instance read from MONIT traces where the flow step index is out
     * of bounds.
     */
    @Test(expected = BuildFlowStepIdxInvalidException.class)
    public void flowStepIndexOutOfBounds() throws URISyntaxException {

        final Project project = this.loadMonitTraces();

        final AssertFlowStep assertFlowStep = new AssertFlowStep();
        assertFlowStep.setFlowStepIdx(1000);

        final AssertFlowInstanceTask afitTask = new AssertFlowInstanceTask();
        afitTask.setRefid(REF_ID);
        afitTask.setFlowInstanceId(FLOW_INSTANCE_ID);
        afitTask.addAssertFlowStep(assertFlowStep);
        afitTask.setProject(project);

        afitTask.execute();

    }

    /**
     * Check assertions on flow steps of a given flow instance read from MONIT traces where a flow step info clause has
     * no property name set.
     */
    @Test(expected = BuildPropertyMissingException.class)
    public void noPropertyOnFlowStepInfo() throws URISyntaxException {

        final Project project = this.loadMonitTraces();

        final AssertFlowStep assertFlowStep = new AssertFlowStep();
        assertFlowStep.setFlowStepIdx(0);
        final FlowStepInfo flowStepInfo_0_flowStepId = new FlowStepInfo();
        flowStepInfo_0_flowStepId.setAttribute("an-attribute");
        assertFlowStep.addFlowStepInfo(flowStepInfo_0_flowStepId);

        final AssertFlowInstanceTask afitTask = new AssertFlowInstanceTask();
        afitTask.setRefid(REF_ID);
        afitTask.setFlowInstanceId(FLOW_INSTANCE_ID);
        afitTask.addAssertFlowStep(assertFlowStep);
        afitTask.setProject(project);

        afitTask.execute();

    }

    /**
     * Check assertions on flow steps of a given flow instance read from MONIT traces where the flow instance is not
     * found
     */
    @Test(expected = BuildFlowInstanceIdInvalidException.class)
    public void flowInstanceNotFound() throws URISyntaxException {

        final Project project = this.loadMonitTraces();

        final AssertFlowStep assertFlowStep = new AssertFlowStep();
        assertFlowStep.setFlowStepIdx(0);

        final AssertFlowInstanceTask afitTask = new AssertFlowInstanceTask();
        afitTask.setRefid(REF_ID);
        afitTask.setFlowInstanceId("a-flow-instance");
        afitTask.addAssertFlowStep(assertFlowStep);
        afitTask.setProject(project);

        afitTask.execute();

    }

    /**
     * Check assertions on flow steps of a given flow instance read from MONIT traces where the ref-id is not as
     * expected
     */
    @Test(expected = BuildRefIdInvalidException.class)
    public void refIdNotAsExpected() throws URISyntaxException {

        final Project project = new Project();
        project.addReference(REF_ID, new Object());

        final AssertFlowInstanceTask afitTask = new AssertFlowInstanceTask();
        afitTask.setRefid(REF_ID);
        afitTask.setProject(project);

        afitTask.execute();

    }
}
