/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task.monit.assertion.flowtreenode;

import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.junit.Test;
import org.ow2.petals.ant.task.monit.FlowTreeNode;
import org.ow2.petals.ant.task.monit.assertion.exception.BuildAssertParamEmptyException;
import org.ow2.petals.ant.task.monit.assertion.exception.BuildAssertParamMissingException;
import org.ow2.petals.ant.task.monit.assertion.flowtreenode.exception.BuildAssertFlowInstanceIdNotEqualsException;
import org.ow2.petals.log.parser.api.model.Flow;
import org.ow2.petals.log.parser.api.model.FlowStep;

/**
 * Unit tests of {@link FlowInstanceIdEquals}.
 * 
 * @author CHristophe DENEUX - Linagora
 *
 */
public class FlowInstanceIdEqualsTest {

    /**
     * Check the assertion against an expected flow instance identifier set to {@code null}.
     */
    @Test(expected = BuildAssertParamMissingException.class)
    public void expectedValueNull() {
        final FlowInstanceIdEquals fiie = new FlowInstanceIdEquals();
        // Don't explicitly set the expected value because of the assertion contained in
        // FlowInstanceIdEquals.setExpected(...);
        fiie.doAssert(new FlowTreeNode(null, null), new Project(), new Location("my-file"));
    }

    /**
     * Check the assertion against an expected flow instance identifier set to an empty value.
     */
    @Test(expected = BuildAssertParamEmptyException.class)
    public void expectedValueEmpty() {
        final FlowInstanceIdEquals fiie = new FlowInstanceIdEquals();
        fiie.setExpected("");
        fiie.doAssert(new FlowTreeNode(null, null), new Project(), new Location("my-file"));
    }

    /**
     * Check the assertion against an expected flow instance identifier set to a not expected value.
     */
    @Test(expected = BuildAssertFlowInstanceIdNotEqualsException.class)
    public void unexpectedValue() {
        final FlowInstanceIdEquals fiie = new FlowInstanceIdEquals();
        fiie.setExpected("unexpected-flow-instance-id");
        fiie.doAssert(new FlowTreeNode(new Flow("flow-instance-id", new FlowStep("step-id")), null), new Project(),
                new Location("my-file"));
    }

    /**
     * Check the assertion against an expected flow instance identifier set to a not expected value.
     */
    public void expectedValue() {
        final FlowInstanceIdEquals fiie = new FlowInstanceIdEquals();
        final String expectedFlowInstanceId = "flow-instance-id";
        fiie.setExpected(expectedFlowInstanceId);
        fiie.doAssert(new FlowTreeNode(new Flow(expectedFlowInstanceId, new FlowStep("step-id")), null), new Project(),
                new Location("my-file"));
    }

}
